#!/usr/bin/env python

import logging
import os
import newparse_mnq as parse_mnq 
import JobListFactory as jlf

job_logger = logging.getLogger("AutoLog.Job")


class Job:
 """Class to handle all the tasks with Jobs at HPC.
    A job is created by default with a name, a jobid, a status and a type.
    It can have children and parents. The inheritance reflects the dependency between jobs.
    If Job2 must wait until Job1 is completed then Job2 is a 'child of Job1. Inversely Job1 is a parent of Job2 """
    
 class Status:
  """Class to handle the status of a job"""
  WAITING = 0
  READY = 1
  SUBMITTED = 2 
  QUEUING = 3
  RUNNING = 4
  COMPLETED = 5
  FAILED = -1

 class JobType:
  """Class to handle the type of a job.
      At the moment contains only 4 types:
      SIMULATION are for multiprocessor jobs
      POSTPROCESSING are single processor jobs
      ClEANING are archiving job---> dealing with large transfer of data on tape
      INITIALISATION are jobs which transfer data from tape to disk"""
  SIMULATION = 0
  POSTPROCESSING = 1
  CLEANING = 2
  INITIALISATION = -1
  
 def __init__(self,name,id,status,jobtype):
  self.name = name
  self.id = id
  self.status = status
  self.jobtype = jobtype
  self.parents = list()
  self.children = list()
  self.failcount=0
  self.expid = 'truc'
 #self.parameters = dict()

 def __eq__(self,other):
  return (self.name==other.name)

 def Print(self):
  print 'NAME: %s' %self.name 
  print 'JOBID: %s' %self.id 
  print 'STATUS: %s' %self.status
  print 'TYPE: %s' %self.jobtype
  print 'PARENTS: %s' %self.parents
  print 'CHILDREN: %s' %self.children
  print 'FAILCOUNTS: %s' %self.failcount
  print 'EXPID: %s' %self.expid 
 
 
 def getName(self):
  return self.name
 
 def getId(self):
  return self.id
 
 def getStatus(self):
  return self.status
 
 def getJobType(self):
  return self.jobtype

 def getExpid(self):
  return self.expid
 
 def getParents(self):
  return self.parents

 def getChildren(self):
  return self.children

 def printJob(self):
  job_logger.info("%s\t%s\t%s" % ("Job Name","Job Id","Job Status"))
  job_logger.info("%s\t\t%s\t%s" % (self.name,self.id,self.status))

 def getAllChildren(self):
  print self.name
  job_list = list()
  if self.hasChildren():
   for job in self.children:
    #job.printJob()
    job_list.append(job)
    print job.getName()
    result_list = job.getAllChildren()
    job_list = job_list+result_list
  job_list=jlf.remove_duplicate(job_list) 
  return job_list

 def getFailCount(self):
  return self.failcount
 
 def setName(self,newName):
  self.name = newName
 
 def setId(self,newId):
  self.id = newId
 
 def setStatus(self,newStatus):
  self.status = newStatus
  
 def setExpid(self,newexpid):
  self.expid = newexpid

 def setJobType(self,newtype):
  self.jobtype = newtype

 def setParents(self,newParents):
  self.parents = newParents
 
 def setChildren(self,newChildren):
  self.children = newChildren
 
 def setFailCount(self,newfailcount):
  self.failcount=newfailcount
 
 def addParents(self,newParents):
  self.parents += [newParents]
 
 def addChildren(self,newChildren):
  self.children += [newChildren]

 def deleteParent(self,parent):
  #careful, it is only possible to remove one parent at a time 
  self.parents.remove(parent)
 
 def deleteChild(self,child):
  # careful it is only possible to remove one child at a time
  self.children.remove(child)

 def hasChildren(self):
  return self.children.__len__() 

 def hasParents(self):
  return self.parents.__len__() 


 def check_completion(self, touched=1):
  complete=False
  parse_mnq.collect()
  logname='../tmp/'+self.name+'_COMPLETED'
  if  (os.path.exists(logname)):
   complete=True
   if touched:
    os.system('touch ../tmp/%s' % self.name+'Checked')
  else:
   job_logger.info("Job: %s has failed!!!" % self.name)
   if touched:
    os.system('touch ../tmp/%s' % self.name+'Failed')
   
  if (complete):
   self.setStatus(Job.Status.COMPLETED)
   if (self.hasChildren()!=0):
    children=self.getChildren()
    job_logger.info("Job is completed, we are now removing the dependency in his %s child/children:" % self.hasChildren())
    for child in children:
     #print "child type.......:",type(child)
     #self.printJob()
     #child.printJob()
     job_logger.debug("number of Parents:",child.hasParents())
     if child.getParents().__contains__(self):
      child.deleteParent(self)

  else:
   job_logger.info("The checking in check_completion tell us that job %s has failed" % self.name)
   self.setStatus(Job.Status.FAILED)


if __name__ == "__main__":
 mainJob = Job('uno','1',Job.Status.READY,0)
 job1 = Job('uno','1',Job.Status.READY,0)
 job2 = Job('dos','2',Job.Status.READY,0)
 job3 = Job('tres','3',Job.Status.READY,0)
 jobs = [job1,job2,job3]
 mainJob.setParents(jobs)
 print mainJob.getParents()
 mainJob.setChildren(jobs)
 print mainJob.getChildren()
 job3.check_completion() 
 print "Number of Parents: ", mainJob.hasParents()
 print "number of children : ", mainJob.hasChildren()
 mainJob.printJob()
